@@ -108,24 +108,51 @@ class Sync:
 
         OUT.debug('Updating selected overlays', 6)
 
+        fatals = []
         warnings = []
         success  = []
         for i in self.selection:
-            ordb = self.rdb.select(i)
-            odb = self.db.select(i)
-            if ordb and odb and ordb.src != odb.src:
-                warnings.append(
-                    'The source of the overlay "' + i + '" seems to have c'
-                    'hanged. You currently sync from "' + odb.src + '" whi'
-                    'le the global layman list reports "' + ordb.src + '" '
-                    'as correct location. Please consider removing and rea'
-                    'dding the overlay!')
+            try:
+                ordb = self.rdb.select(i)
+            except:
+                warnings.append(\
+                    'Overlay "%s" could not be found in the remote lists.\n'
+                    'Please check if it has been renamed and re-add if necessary.' % i)
+            else:
+                odb = self.db.select(i)
+                current_src = odb.sources[0].src
+                available_srcs = set(e.src for e in ordb.sources)
+                if ordb and odb and not current_src in available_srcs:
+                    if len(available_srcs) == 1:
+                        plural = ''
+                        candidates = '  %s' % tuple(available_srcs)[0]
+                    else:
+                        plural = 's'
+                        candidates = '\n'.join(('  %d. %s' % (i + 1, v)) for i, v in enumerate(available_srcs))
+
+                    warnings.append(
+                        'The source of the overlay "%(repo_name)s" seems to have changed.\n'
+                        'You currently sync from\n'
+                        '\n'
+                        '  %(current_src)s\n'
+                        '\n'
+                        'while the remote lists report\n'
+                        '\n'
+                        '%(candidates)s\n'
+                        '\n'
+                        'as correct location%(plural)s.\n'
+                        'Please consider removing and re-adding the overlay.' % {
+                            'repo_name':i,
+                            'current_src':current_src,
+                            'candidates':candidates,
+                            'plural':plural,
+                            })
 
             try:
                 self.db.sync(i, self.quiet)
                 success.append('Successfully synchronized overlay "' + i + '".')
             except Exception, error:
-                warnings.append(
+                fatals.append(
                     'Failed to sync overlay "' + i + '".\nError was: '
                     + str(error))
 
@@ -135,9 +162,14 @@ class Sync:
                 OUT.info(i, 3)
                 
         if warnings:
-            OUT.warn('\nErrors:\n------\n', 2)
+            OUT.warn('\nWarnings:\n------\n', 2)
             for i in warnings:
                 OUT.warn(i + '\n', 2)
+
+        if fatals:
+            OUT.error('\nErrors:\n------\n')
+            for i in fatals:
+                OUT.error(i + '\n')
             return 1
 
         return 0
@@ -275,6 +307,7 @@ class Info:
     * Source  : https://overlays.gentoo.org/svn/dev/wrobel
     * Contact : nobody@gentoo.org
     * Type    : Subversion; Priority: 10
+    * Quality : experimental
     * 
     * Description:
     *   Test
@@ -355,6 +388,7 @@ class List:
     * Source  : https://overlays.gentoo.org/svn/dev/wrobel
     * Contact : nobody@gentoo.org
     * Type    : Subversion; Priority: 10
+    * Quality : experimental
     * 
     * Description:
     *   Test
@@ -366,6 +400,7 @@ class List:
     * Source  : rsync://gunnarwrobel.de/wrobel-stable
     * Contact : nobody@gentoo.org
     * Type    : Rsync; Priority: 50
+    * Quality : experimental
     * 
     * Description:
     *   A collection of ebuilds from Gunnar Wrobel [wrobel@gentoo.org].
