@@ -265,6 +265,14 @@ class RemoteDB(Overlays):
 
             mpath = self.path(url)
 
+            # Check for sufficient privileges
+            if os.path.exists(mpath) and not os.access(mpath, os.W_OK):
+                OUT.warn('You do not have permission to update the cache (%s).' % mpath)
+                import getpass
+                if getpass.getuser() != 'root':
+                    OUT.warn('Hint: You are not root.')
+                continue
+
             try:
 
                 # Fetch the remote list
@@ -343,7 +351,7 @@ class MakeConf:
     ...           here + '/tests/testfiles/global-overlays.xml',
     ...           'make_conf' : here + '/tests/testfiles/make.conf',
     ...           'nocheck'    : True,
-    ...           'storage'   : '/usr/local/portage/layman',
+    ...           'storage'   : '/var/lib/layman',
     ...           'quietness':3}
     >>> b = DB(config)
     >>> a = MakeConf(config, b.overlays)
@@ -391,7 +399,7 @@ class MakeConf:
         ...           here + '/tests/testfiles/global-overlays.xml',
         ...           'make_conf' : here + '/tests/testfiles/make.conf',
         ...           'nocheck'    : True,
-        ...           'storage'   : '/usr/local/portage/layman',
+        ...           'storage'   : '/var/lib/layman',
         ...           'quietness':3}
         >>> c = DB(config)
         >>> a = MakeConf(config, c.overlays)
@@ -419,7 +427,7 @@ class MakeConf:
         ...           here + '/tests/testfiles/global-overlays.xml',
         ...           'make_conf' : here + '/tests/testfiles/make.conf',
         ...           'nocheck'    : True,
-        ...           'storage'   : '/usr/local/portage/layman',
+        ...           'storage'   : '/var/lib/layman',
         ...           'quietness':3}
         >>> c = DB(config)
         >>> a = MakeConf(config, c.overlays)
@@ -448,7 +456,7 @@ class MakeConf:
         ...           here + '/tests/testfiles/global-overlays.xml',
         ...           'make_conf' : here + '/tests/testfiles/make.conf',
         ...           'nocheck'    : True,
-        ...           'storage'   : '/usr/local/portage/layman',
+        ...           'storage'   : '/var/lib/layman',
         ...           'quietness':3}
         >>> c = DB(config)
         >>> a = MakeConf(config, c.overlays)
@@ -503,7 +511,7 @@ class MakeConf:
         ...           here + '/tests/testfiles/global-overlays.xml',
         ...           'make_conf' : here + '/tests/testfiles/make.conf',
         ...           'nocheck'    : True,
-        ...           'storage'   : '/usr/local/portage/layman',
+        ...           'storage'   : '/var/lib/layman',
         ...           'quietness':3}
         >>> c = DB(config)
         >>> a = MakeConf(config, c.overlays)
