@@ -31,34 +31,9 @@ __version__ = "$Id: overlay.py 273 2006-12-30 15:54:50Z wrobel $"
 import sys, codecs, os, os.path
 import xml.etree.ElementTree as ET # Python 2.5
 
-from   layman.overlays.bzr       import BzrOverlay
-from   layman.overlays.darcs     import DarcsOverlay
-from   layman.overlays.git       import GitOverlay
-from   layman.overlays.mercurial import MercurialOverlay
-from   layman.overlays.cvs       import CvsOverlay
-from   layman.overlays.svn       import SvnOverlay
-from   layman.overlays.rsync     import RsyncOverlay
-from   layman.overlays.tar       import TarOverlay
-
 from   layman.debug              import OUT
 from   layman.utils              import indent
-
-#===============================================================================
-#
-# Constants
-#
-#-------------------------------------------------------------------------------
-
-OVERLAY_TYPES = dict((e.type_key, e) for e in (
-    GitOverlay,
-    CvsOverlay,
-    SvnOverlay,
-    RsyncOverlay,
-    TarOverlay,
-    BzrOverlay,
-    MercurialOverlay,
-    DarcsOverlay
-))
+from   layman.overlays.overlay   import Overlay
 
 #===============================================================================
 #
@@ -115,8 +90,8 @@ class Overlays:
         >>> a.overlays.keys()
         [u'wrobel', u'wrobel-stable']
 
-        >>> a.overlays['wrobel-stable'].src
-        u'rsync://gunnarwrobel.de/wrobel-stable'
+        >>> list(a.overlays['wrobel-stable'].source_uris())
+        [u'rsync://gunnarwrobel.de/wrobel-stable']
         '''
         document = ET.fromstring(text)
         overlays = document.findall('overlay') + \
@@ -124,29 +99,12 @@ class Overlays:
 
         for overlay in overlays:
             OUT.debug('Parsing overlay entry', 8)
-            
-            _source = overlay.find('source')
-            if _source != None:
-                element_to_scan = _source
-            else:
-                element_to_scan = overlay
+            try:
+                ovl = Overlay(overlay, self.config, self.ignore, self.quiet)
+                self.overlays[ovl.name] = ovl
+            except Exception, error:
+                OUT.warn(str(error), 3)
 
-            for attr_name in element_to_scan.attrib.keys():
-                if attr_name == 'type':
-                    overlay_type = element_to_scan.attrib['type']
-                    if overlay_type in OVERLAY_TYPES.keys():
-                        try:
-                            ovl = OVERLAY_TYPES[overlay_type](overlay,
-                                                                self.config,
-                                                                self.ignore,
-                                                                self.quiet)
-                            self.overlays[ovl.name] = ovl
-                        except Exception, error:
-                            OUT.warn(str(error), 3)
-                    else:
-                        raise Exception('Unknown overlay type "' +
-                                        overlay_type + '"!')
-                    break
 
     def write(self, path):
         '''
@@ -188,11 +146,12 @@ class Overlays:
         >>> here = os.path.dirname(os.path.realpath(__file__))
         >>> config = {'svn_command': '/usr/bin/svn', 'rsync_command':'/usr/bin/rsync'}
         >>> a = Overlays([here + '/tests/testfiles/global-overlays.xml', ], config)
-        >>> a.select('wrobel-stable').src
-        u'rsync://gunnarwrobel.de/wrobel-stable'
+        >>> list(a.select('wrobel-stable').source_uris())
+        [u'rsync://gunnarwrobel.de/wrobel-stable']
         '''
-        if overlay in self.overlays.keys():
-            return self.overlays[overlay]
+        if not overlay in self.overlays.keys():
+            raise Exception('No overlay "%s" in database' % overlay)
+        return self.overlays[overlay]
 
     def list(self, verbose = False, width = 0):
         '''
@@ -208,6 +167,7 @@ class Overlays:
         Source  : https://overlays.gentoo.org/svn/dev/wrobel
         Contact : nobody@gentoo.org
         Type    : Subversion; Priority: 10
+        Quality : experimental
         <BLANKLINE>
         Description:
           Test
@@ -217,6 +177,7 @@ class Overlays:
         Source  : rsync://gunnarwrobel.de/wrobel-stable
         Contact : nobody@gentoo.org
         Type    : Rsync; Priority: 50
+        Quality : experimental
         <BLANKLINE>
         Description:
           A collection of ebuilds from Gunnar Wrobel [wrobel@gentoo.org].
