@@ -26,7 +26,7 @@ __version__ = "$Id: mercurial.py 236 2006-09-05 20:39:37Z wrobel $"
 #-------------------------------------------------------------------------------
 
 from   layman.utils             import path
-from   layman.overlays.overlay  import Overlay
+from   layman.overlays.source   import OverlaySource
 
 #===============================================================================
 #
@@ -34,15 +34,15 @@ from   layman.overlays.overlay  import Overlay
 #
 #-------------------------------------------------------------------------------
 
-class MercurialOverlay(Overlay):
+class MercurialOverlay(OverlaySource):
     ''' Handles mercurial overlays.'''
 
     type = 'Mercurial'
     type_key = 'mercurial'
 
-    def __init__(self, xml, config, ignore = 0, quiet = False):
+    def __init__(self, parent, xml, config, _location, ignore = 0, quiet = False):
 
-        Overlay.__init__(self, xml, config, ignore)
+        super(MercurialOverlay, self).__init__(parent, xml, config, _location, ignore, quiet)
 
     def add(self, base, quiet = False):
         '''Add overlay.'''
@@ -50,18 +50,18 @@ class MercurialOverlay(Overlay):
         self.supported()
 
         return self.cmd(self.command() + ' clone "' + self.src + '/" "' +
-                        path([base, self.name]) + '"')
+                        path([base, self.parent.name]) + '"')
 
     def sync(self, base, quiet = False):
         '''Sync overlay.'''
 
         self.supported()
 
-        return self.cmd('cd "' + path([base, self.name]) + '" && ' +
+        return self.cmd('cd "' + path([base, self.parent.name]) + '" && ' +
                         self.command() + ' pull -u "' + self.src + '"')
 
     def supported(self):
         '''Overlay type supported?'''
 
-        return Overlay.supported(self, [(self.command(),  'mercurial',
+        return super(MercurialOverlay, self).supported([(self.command(),  'mercurial',
                                          'dev-util/mercurial'),])
