@@ -28,7 +28,7 @@ __version__ = "$Id: overlay.py 273 2006-12-30 15:54:50Z wrobel $"
 #
 #-------------------------------------------------------------------------------
 
-import sys, types, re, os, os.path, shutil, subprocess
+import sys, re, os, os.path
 import codecs
 import locale
 import xml.etree.ElementTree as ET # Python 2.5
@@ -37,6 +37,36 @@ from   layman.utils             import path, ensure_unicode
 
 from   layman.debug             import OUT
 
+from   layman.overlays.bzr       import BzrOverlay
+from   layman.overlays.darcs     import DarcsOverlay
+from   layman.overlays.git       import GitOverlay
+from   layman.overlays.mercurial import MercurialOverlay
+from   layman.overlays.cvs       import CvsOverlay
+from   layman.overlays.svn       import SvnOverlay
+from   layman.overlays.rsync     import RsyncOverlay
+from   layman.overlays.tar       import TarOverlay
+
+#===============================================================================
+#
+# Constants
+#
+#-------------------------------------------------------------------------------
+
+OVERLAY_TYPES = dict((e.type_key, e) for e in (
+    GitOverlay,
+    CvsOverlay,
+    SvnOverlay,
+    RsyncOverlay,
+    TarOverlay,
+    BzrOverlay,
+    MercurialOverlay,
+    DarcsOverlay
+))
+
+QUALITY_LEVELS = 'core|stable|testing|experimental|graveyard'.split('|')
+
+WHITESPACE_REGEX = re.compile('\s+')
+
 #===============================================================================
 #
 # Class Overlay
@@ -46,8 +76,6 @@ from   layman.debug             import OUT
 class Overlay(object):
     ''' Derive the real implementations from this.'''
 
-    type = 'None'
-
     def __init__(self, xml, config, ignore = 0, quiet = False):
         '''
         >>> here = os.path.dirname(os.path.realpath(__file__))
@@ -59,8 +87,8 @@ class Overlay(object):
         u'wrobel'
         >>> a.is_official()
         True
-        >>> a.src
-        u'https://overlays.gentoo.org/svn/dev/wrobel'
+        >>> list(a.source_uris())
+        [u'https://overlays.gentoo.org/svn/dev/wrobel']
         >>> a.owner_email
         u'nobody@gentoo.org'
         >>> a.description
@@ -71,8 +99,6 @@ class Overlay(object):
         >>> b.is_official()
         False
         '''
-        self.config = config
-        self.quiet = quiet
 
         _name = xml.find('name')
         if _name != None:
@@ -82,14 +108,30 @@ class Overlay(object):
         else:
             raise Exception('Overlay is missing a "name" entry!')
 
-        _source = xml.find('source')
-        if _source == None:
-            if 'src' in xml.attrib:
-                _source = ET.Element('source')
-                _source.text = xml.attrib['src']
-            else:
-                raise Exception('Overlay "' + self.name + '" is missing a "source" entry!')
-        self._set_source(_source)
+        _sources = xml.findall('source')
+        if _sources:
+            _sources = [e for e in _sources if 'type' in e.attrib]
+        elif ('src' in xml.attrib) and ('type' in xml.attrib):
+                s = ET.Element('source', type=xml.attrib['type'])
+                s.text = xml.attrib['src']
+                _sources = [s]
+                del s
+
+        if not _sources:
+            raise Exception('Overlay "' + self.name + '" is missing a "source" entry!')
+
+
+        def create_overlay_source(source_elem):
+            _type = source_elem.attrib['type']
+            try:
+                _class = OVERLAY_TYPES[_type]
+            except KeyError:
+                raise Exception('Unknown overlay type "%s"!' % _type)
+            _location = ensure_unicode(source_elem.text.strip())
+            return _class(self, xml, config, _location, ignore, quiet)
+
+        self.sources = map(create_overlay_source, _sources)
+
 
         _owner = xml.find('owner')
         if _owner == None:
@@ -119,7 +161,9 @@ class Overlay(object):
 
         _desc = xml.find('description')
         if _desc != None:
-            self.description = ensure_unicode(_desc.text.strip())
+            d = WHITESPACE_REGEX.sub(' ', _desc.text.strip())
+            self.description = ensure_unicode(d)
+            del d
         else:
             self.description = ''
             if not ignore:
@@ -134,6 +178,11 @@ class Overlay(object):
         else:
             self.status = None
 
+        self.quality = u'experimental'
+        if 'quality' in xml.attrib:
+            if xml.attrib['quality'] in set(QUALITY_LEVELS):
+                self.quality = ensure_unicode(xml.attrib['quality'])
+
         if 'priority' in xml.attrib:
             self.priority = int(xml.attrib['priority'])
         else:
@@ -148,19 +197,24 @@ class Overlay(object):
         else:
             self.homepage = None
 
+        self.feeds = [ensure_unicode(e.text.strip()) for e in xml.findall('feed')]
+
+
     def __eq__(self, other):
         for i in ('description', 'homepage', 'name', 'owner_email',
-                'owner_name', 'priority', 'src', 'status'):
+                'owner_name', 'priority', 'status'):
             if getattr(self, i) != getattr(other, i):
                 return False
+        for i in self.sources + other.sources:
+            if not i in self.sources:
+                return False
+            if not i in other.sources:
+                return False
         return True
 
     def __ne__(self, other):
         return not self.__eq__(other)
 
-    def _set_source(self, source_elem):
-        self.src = ensure_unicode(source_elem.text.strip())
-
     def set_priority(self, priority):
         '''Set the priority of this overlay.'''
 
@@ -172,6 +226,7 @@ class Overlay(object):
         repo = ET.Element('repo')
         if self.status != None:
             repo.attrib['status'] = self.status
+        repo.attrib['quality'] = self.quality
         repo.attrib['priority'] = str(self.priority)
         name = ET.Element('name')
         name.text = self.name
@@ -192,55 +247,42 @@ class Overlay(object):
             owner_name = ET.Element('name')
             owner_name.text = self.owner_name
             owner.append(owner_name)
-        source = ET.Element('source', type=self.__class__.type_key)
-        source.text = self.src
-        repo.append(source)
+        for i in self.sources:
+            source = ET.Element('source', type=i.__class__.type_key)
+            source.text = i.src
+            repo.append(source)
+            del source
+        for i in self.sources:
+            # NOTE: Two loops on purpose so the
+            # hooks are called with all sources in
+            i.to_xml_hook(repo)
+        for i in self.feeds:
+            feed = ET.Element('feed')
+            feed.text = i
+            repo.append(feed)
+            del feed
         return repo
 
     def add(self, base, quiet = False):
-        '''Add the overlay.'''
-
-        mdir = path([base, self.name])
-
-        if os.path.exists(mdir):
-            raise Exception('Directory ' + mdir + ' already exists. Will not ov'
-                            'erwrite its contents!')
-
-        os.makedirs(mdir)
+        res = 1
+        for s in self.sources:
+            try:
+                res = s.add(base, quiet)
+                if res == 0:
+                    # Worked, throw other sources away
+                    self.sources = [s]
+                    break
+            except Exception, error:
+                OUT.warn(str(error), 4)
+        return res
 
     def sync(self, base, quiet = False):
-        '''Sync the overlay.'''
-        pass
+        assert len(self.sources) == 1
+        return self.sources[0].sync(base, quiet)
 
     def delete(self, base):
-        '''Delete the overlay.'''
-        mdir = path([base, self.name])
-
-        if not os.path.exists(mdir):
-            OUT.warn('Directory ' + mdir + ' did not exist, no files deleted.')
-            return
-
-        shutil.rmtree(mdir)
-
-    def cmd(self, command):
-        '''Run a command.'''
-
-        OUT.info('Running command "' + command + '"...', 2)
-
-        if hasattr(sys.stdout,'encoding'):
-            enc = sys.stdout.encoding or sys.getfilesystemencoding()
-            if enc:
-                command = command.encode(enc)
-
-        if not self.quiet:
-            return os.system(command)
-        else:
-            cmd = subprocess.Popen([command], shell = True,
-                                   stdout = subprocess.PIPE,
-                                   stderr = subprocess.PIPE,
-                                   close_fds = True)
-            result = cmd.wait()
-            return result
+        assert len(self.sources) == 1
+        return self.sources[0].delete(base)
 
     def _get_encoding(self):
         if hasattr(sys.stdout, 'encoding') \
@@ -264,7 +306,8 @@ class Overlay(object):
         ~~~~~~
         Source  : https://overlays.gentoo.org/svn/dev/wrobel
         Contact : nobody@gentoo.org
-        Type    : None; Priority: 10
+        Type    : Subversion; Priority: 10
+        Quality : experimental
         <BLANKLINE>
         Description:
           Test
@@ -275,13 +318,25 @@ class Overlay(object):
 
         result += self.name + u'\n' + (len(self.name) * u'~')
 
-        result += u'\nSource  : ' + self.src
+        if len(self.sources) == 1:
+            result += u'\nSource  : ' + self.sources[0].src
+        else:
+            result += u'\nSources:'
+            for i, v in enumerate(self.sources):
+                result += '\n  %d. %s' % (i + 1, v.src)
+            result += '\n'
+
         if self.owner_name != None:
             result += u'\nContact : %s <%s>' % (self.owner_name, self.owner_email)
         else:
             result += u'\nContact : ' + self.owner_email
-        result += u'\nType    : ' + self.type
+        if len(self.sources) == 1:
+            result += u'\nType    : ' + self.sources[0].type
+        else:
+            result += u'\nType    : ' + '/'.join(sorted(set(e.type for e in self.sources)))
         result += u'; Priority: ' + str(self.priority) + u'\n'
+        result += u'Quality : ' + self.quality + u'\n'
+
 
         description = self.description
         description = re.compile(u' +').sub(u' ', description)
@@ -294,10 +349,16 @@ class Overlay(object):
             link = self.homepage
             link = re.compile(u' +').sub(u' ', link)
             link = re.compile(u'\n ').sub(u'\n', link)
-            result += u'\nLink:\n'
+            result += u'\nLink:'
             result += u'\n  '.join((u'\n' + link).split(u'\n'))
             result += u'\n'
 
+        if self.feeds:
+            result += u'\n%s:' % ((len(self.feeds) == 1) and "Feed" or "Feeds")
+            for i in self.feeds:
+               result += u'\n  %s' % i
+            result += u'\n'
+
         return self._encode(result)
 
     def short_list(self, width = 0):
@@ -308,7 +369,7 @@ class Overlay(object):
         >>> overlays = document.findall('repo') + document.findall('overlay')
         >>> a = Overlay(overlays[0], dict())
         >>> print a.short_list(80)
-        wrobel                    [None      ] (https://o.g.o/svn/dev/wrobel         )
+        wrobel                    [Subversion] (https://o.g.o/svn/dev/wrobel         )
         '''
 
         def pad(string, length):
@@ -338,57 +399,34 @@ class Overlay(object):
             return 80
 
         name   = pad(self.name, 25)
-        mtype  = ' [' + pad(self.type, 10) + ']'
+
+        if len(set(e.type for e in self.sources)) == 1:
+            _type = self.sources[0].type
+        else:
+            _type = '%s/..' % self.sources[0].type
+
+        mtype  = ' [' + pad(_type, 10) + ']'
         if not width:
             width = terminal_width()
         srclen = width - 43
-        source = self.src
+        source = ', '.join(self.source_uris())
         if len(source) > srclen:
             source = source.replace("overlays.gentoo.org", "o.g.o")
         source = ' (' + pad(source, srclen) + ')'
 
         return self._encode(name + mtype + source)
 
-    def supported(self, binaries = []):
-        '''Is the overlay type supported?'''
-
-        if binaries:
-            for command, mtype, package in binaries:
-                found = False
-                if os.path.isabs(command):
-                    kind = 'Binary'
-                    found = os.path.exists(command)
-                else:
-                    kind = 'Command'
-                    for d in os.environ['PATH'].split(os.pathsep):
-                        f = os.path.join(d, command)
-                        if os.path.exists(f):
-                            found = True
-                            break
-
-                if not found:
-                    raise Exception(kind + ' ' + command + ' seems to be missing!'
-                                    ' Overlay type "' + mtype + '" not support'
-                                    'ed. Did you emerge ' + package + '?')
-
-        return True
-
-    def is_supported(self):
-        '''Is the overlay type supported?'''
-
-        try:
-            self.supported()
-            return True
-        except Exception, error:
-            return False
-
     def is_official(self):
         '''Is the overlay official?'''
 
         return self.status == 'official'
 
-    def command(self):
-        return self.config['%s_command' % self.__class__.type_key]
+    def is_supported(self):
+        return any(e.is_supported() for e in self.sources)
+
+    def source_uris(self):
+        for i in self.sources:
+            yield i.src
 
 #================================================================================
 #
